package docar.archive.document;

import java.net.URL;
import java.sql.Date;
import java.util.Calendar;
import java.util.Set;

import docar.user.Writer;
import docar.util.profile.Profiler;
import docar.util.profile.StopWatch;

/**
 * '문서(Document)' 객체를 생성하는 빌더(builder) 클래스.
 * 
 * @author "Sunny Kwak"
 *
 */
public class DocumentBuilder {
	private URL location;
	private Writer writer;
	private Date createDate;
	private String title;
	private String description;
	private Set<String> keywords;
	
	/**
	 * 문서의 물리적인 파일 위치 (URL), 필수 항목.
	 * @param location 파일 위치
	 */
	public void setLocation(URL location) {
		this.location = location;
	}
	
	/**
	 * 문서 작성자 정보, 필수 항목.
	 * 
	 * @param writer 문서 작성자 
	 */
	public void setWriter(Writer writer) {
		this.writer = writer;
	}
	
	/**
	 * 문서 생성 일시, 입력하지 않을 경우 현재 시간으로 설정.
	 * 
	 * @param createDate 문서 생성 일시
	 */
	public void setCreateDate(Date createDate) {
		this.createDate = createDate;
	}

	/**
	 * 문서 제목,
	 * @param title 문서 제목
	 */
	public void setTitle(String title) {
		this.title = title;
	}
	
	public void setDescription(String description) {
		this.description = description;
	}
	
	public void setKeywords(Set<String> keywords) {
		this.keywords = keywords;
	}
	
	
	public Document createDocument() {
		
		StopWatch stopWatch = new StopWatch("DocumentBuilder.createDocument");

		// ---- 필수 항목 점검 및 analyzer 준비 ----
		if(location == null) {
			throw new IllegalStateException("location property is not specified");
		}
		if(writer == null) {
			throw new IllegalStateException("writer property is not specified");
		}
		
		ContentAnalyzer contentAnalyzer = new ContentAnalyzer(location);
		contentAnalyzer.analyze();

		// ---- 입력 값이 없는 항목에 대한 대체 처리 -----
		if(createDate == null) {
			createDate = new Date(Calendar.getInstance().getTimeInMillis());
		}
		if(title == null) {
			title = contentAnalyzer.getFileName();
		}

		// -- FileVersion, File, Document 순서로 빌드 -----
		FileVersion fileVersion = new FileVersion(1, createDate, description);
		File file = new File(location, contentAnalyzer.getChecksum(), contentAnalyzer.getSize(), fileVersion);
		Document document = new Document(title, writer, keywords, contentAnalyzer.getAbstraction(), location, contentAnalyzer.getFormat());
		document.addFile(file);
		
		Profiler.getInstance().addCallEvent(stopWatch.stop());
		
		return document;
	}
}
