package docar.storage.volume;

import java.io.File;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.ArrayList;
import java.util.List;
import java.util.Stack;

/**
 * 디스크 볼륨.
 *
 * @author "Sunny Kwak"
 */
public class DiskVolume implements Volume {

	private File rootFile;
	private Stack<DirectoryScan> directoryStack;
	private DirectoryScan workingDirectoryScan;
	private List<Filter> filterList;

	public DiskVolume(File rootFile) {
		this.rootFile = rootFile;
	}

	@Override
	public URL getURL() {
		try {
			return rootFile.toURI().toURL();
		} catch (MalformedURLException ignored) {
			// do nothing
		}

		return null;
	}

	@Override
	public void addFilter(Filter filter) {
		if (filterList == null) {
			filterList = new ArrayList<Filter>();
		}
		filterList.add(filter);
	}

	@Override
	public void open() {
		if (!rootFile.canRead() || !rootFile.isDirectory()) {
			throw new IllegalStateException(
					"Cannot read file system or it's not a directory : "
							+ rootFile.getAbsolutePath());
		}

		// 최상위 디렉토리를 스택에 넣고 스캔 준비.
		directoryStack = new Stack<DirectoryScan>();
		workingDirectoryScan = new DirectoryScan(rootFile, rootFile.listFiles());
	}

	@Override
	public URL nextFileURL() {

		if (directoryStack == null) {
			throw new IllegalStateException("Error : not opened");
		}

		URL url = null;

		do {
			// 현재 디렉토리에서 다음 파일 선택
			File nextFile = null;
			do {
				nextFile = workingDirectoryScan.getNextFile();
			} while (nextFile != null && !doFilter(nextFile));

			// 다음 파일이 없다면...
			if (nextFile == null) {
				// 스택에 상위 디렉토리가 남아 있다면, 상위 디렉토리를 꺼낸다.
				if (!directoryStack.empty()) {
					workingDirectoryScan = directoryStack.pop();
				}
				// 다음 파일도 없고, 상위 디렉토리도 없다면, 종료.
				else {
					break;
				}
			}
			// 다음 파일이 디렉토리라면, 스택에 넣고 루프 반복.
			else if (nextFile.isDirectory()) {
				directoryStack.push(workingDirectoryScan);
				workingDirectoryScan = new DirectoryScan(nextFile,
						nextFile.listFiles());
			}
			// 다음 파일이 디렉토리가 아니라면, 반환..
			else {
				try {
					return nextFile.toURI().toURL();
				} catch (MalformedURLException ignored) {
					// do nothing
				}
			}
		}
		// 1) 반환할 파일 (다음 파일)이 존재하면 스캔 종료, 2) 반환할 파일이 없으면, 디렉토리를 다시 탐색한다.
		while (url == null);

		return url;
	}

	private boolean doFilter(File file) {
		for (Filter filter : filterList) {
			if (!filter.allowed(file))
				return false;
		}
		return true;
	}

	@Override
	public void close() {
		directoryStack = null;
		workingDirectoryScan = null;
	}

	/**
	 * 특정(단일) 디렉토리의 파일 목록을 보관하며, 해당 디렉토리의 파일들을 하나씩 반환한다.
	 */
	class DirectoryScan {
		private File directory;
		private File[] filesInDirectory;
		private int index;

		DirectoryScan(File directory, File[] filesInDirectory) {
			this.directory = directory;
			this.filesInDirectory = filesInDirectory;
			index = 0;
		}

		File getDirectory() {
			return directory;
		}

		File getNextFile() {
			return (filesInDirectory != null && index < filesInDirectory.length) ? filesInDirectory[index++]
					: null;
		}
	}

}
