package docar.util.profile;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.Map.Entry;

/**
 * 성능 측정 클래스.
 *
 * @author "Sunny Kwak"
 */
public class Profiler {
	
	private static Profiler singleton;
	private Map<String, Accumulation> profilingMap;


	/**
	 * 싱글턴(singleton) 패턴의 클래스이므로, 공개 생성자를 제공하지 않는다.
	 */
	Profiler() {
		profilingMap = new HashMap<String, Accumulation>();
	}

	/**
	 * @return Profiler 싱글턴 인스턴스 반환
	 */
	public static Profiler getInstance() {
		if(singleton == null) {
			singleton = new Profiler();
		}
		
		return singleton;
	}
	
	/**
	 * 기능 호출 이벤트를 기록한다.
	 * 
	 * @param stopWatch 기능 수행 시간 정보를 포함하는 인스턴스.
	 */
	public void addCallEvent(StopWatch stopWatch) {
		String functionName = stopWatch.getFunctionName();
		Accumulation accumulation = profilingMap.get(functionName);
		if(accumulation == null) {
			accumulation = new Accumulation();
			profilingMap.put(functionName, accumulation);
		}
		
		accumulation.add(stopWatch.getEllapsedTime());
	}
	
	/**
	 * @return 누적된 수행 시간 기록을 반환
	 */
	public Iterator<Entry<String, Accumulation>> iterator() {
		return profilingMap.entrySet().iterator();
	}

	/**
	 * 개별 기능에 대한 수행 기록 (호출 횟수, 총 수행 시간)을 기록하는 클래스
	 */
	class Accumulation {
		// 기능 호출 횟수
		private long callNumber;
		// 수행 시간
		private long accumaltedRunningTime;
		
		public Accumulation() {
			callNumber = 0;
			accumaltedRunningTime = 0L;
		}
		
		/**
		 * 호출 횟수 및 수행 시간 누적.
		 * 
		 * @param runningTime 1회 수행 시간.
		 */
		public void add(long runningTime) {
			callNumber ++;
			accumaltedRunningTime += runningTime;
		}
		
		/**
		 * @return 누적된 호출 횟수 반환
		 */
		public long getCallNumber() {
			return callNumber;
		}
		
		/**
		 * @return 누적된 실행시간 반환
		 */
		public long getAccumulatedRunningTime() {
			return accumaltedRunningTime;
		}
	}
}
